from Icon import Icon
from utils import FileWatcher

import commands
import os


class IconSet:

    # the editor for starters
    __EDITOR = "gnome-desktop-item-edit"

    # where the starters are stored
    __ICONPATH = os.path.expanduser("~/.gdesklets/MyStarters")

    # the file containing the ordering of starters
    __ORDERFILE = os.path.join(__ICONPATH, ".order")


    def __init__(self):

        # the icons
        self.__icons = []

        # the order of the icons
        self.__order = []

        # the visualizer for icons
        self.__visualizer = None


        self.__initialize()
        watcher = FileWatcher(self.__ORDERFILE)
        watcher.add_observer(self.__on_observe_icons)




    #
    # Initializes the icon set, if necessary.
    #
    def __initialize(self):

        # create icon dir, if necessary
        if (not os.path.exists(self.__ICONPATH)):
            os.makedirs(self.__ICONPATH)

        # create order file, if necessary
        if (not os.path.exists(self.__ORDERFILE)):
            cwd = os.getcwd()
            iconpath = self.__ICONPATH
            file1 = os.path.join(cwd, "home.desktop")
            file2 = os.path.join(cwd, ".order")
            os.system("cp %(file1)s %(file2)s %(iconpath)s/" % vars())
            #os.system("touch " + self.__ORDERFILE)
            




    #
    # Reacts on changes of the icons.
    #
    def __on_observe_icons(self, src, cmd, *args):

        self.__order = self.__load_order()

        self.__icons = []
        for o in self.__order[:]:
            path = os.path.join(self.__ICONPATH, o)
            # remove stale filenames
            if (not os.path.exists(path)):
                self.__order.remove(o)
                continue

            icon = Icon(path)
            self.__icons.append(icon)
        #end for

        if (self.__visualizer): self.__visualizer(self)



    #
    # Loads the order of icons.
    #
    def __load_order(self):

        order = []

        fd = open(self.__ORDERFILE, "r")
        for i in fd.readlines():
            # ignore empty lines
            if (i == "\n"): continue
            
            # drop the trailing newline
            if (i[-1] == "\n"): order.append(i[:-1])
            else: order.append(i)
        fd.close()

        return order



    #
    # Stores the order of icons.
    #
    def __store_order(self, order):

        fd = open(self.__ORDERFILE, "w")
        fd.write("\n".join(order))
        fd.write("\n")
        fd.close()



    #
    # Forces loading the icons manually.
    #
    def load_icons(self):

        self.__on_observe_icons(None, None)



    #
    # Adds a new icon and opens the icon editor.
    #
    def add_icon(self):

        name = commands.getoutput("uuidgen") + ".desktop"
        path = os.path.join(self, self.__ICONPATH, name)

        cmd = self.__EDITOR + " " + path + " && " + \
              "echo " + name + " >>" + self.__ORDERFILE + " &"
        os.system(cmd)

        self.__order.append(name)

    


    #
    # Removes the icon at the given position.
    #
    def remove_icon(self, index):

        if (index == -1): return

        name = self.__order[index]
        path = os.path.join(self.__ICONPATH, name)
        os.unlink(path)

        self.__order.remove(name)
        self.__store_order(self.__order)
        print "removed"



    #
    # Opens an editor for the given icon.
    #
    def edit_icon(self, index):

        if (index == -1): return

        name = self.__order[index]
        path = os.path.join(self.__ICONPATH, name)

        cmd = self.__EDITOR + " " + path + " && " + \
              "touch " + self.__ORDERFILE + " &"
        os.system(cmd)



    #
    # Swaps the two given icons.
    #
    def swap_icons(self, index1, index2):

        tmp = self.__order[index2]
        self.__order[index2] = self.__order[index1]
        self.__order[index1] = tmp
        self.__store_order(self.__order)



    #
    # Returns the icon at the given position index.
    #
    def get_icon(self, index):

        return self.__icons[index]



    #
    # Returns the length of the icon set, i.e. the number of the icons in it.
    #
    def get_length(self):

        return len(self.__icons)



    #
    # Sets the visualizer function for the icon set. The visualizer is called
    # whenever the icon set changes.
    #
    def set_visualizer(self, handler):

        self.__visualizer = handler
